'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Merge to objects together
 *
 * If the target and source both have the same key, target overwrites
 * the source, otherwise leave the source key in place.
 *
 * This is a fairly simple implimentation and may break on some edge cases.
 *
 * @param  {Object} target Values that should overwrite defaults
 * @param  {Object} source Default values to fallback to
 * @return {Object}        Merged target + source
 */
var deepObjectExtend = function deepObjectExtend(defaults, options) {
    for (var prop in options) {
        if (options.hasOwnProperty(prop)) {
            if (defaults[prop] && _typeof(options[prop]) === 'object') {
                deepObjectExtend(defaults[prop], options[prop]);
            } else {
                defaults[prop] = options[prop];
            }
        }
    }
    return defaults;
};

/**
 * Take a list of gulp plugins and a reference to the instance of gulp when
 * importing the function, so we don't have to reference it everytime
 * we call task() in the gulpfile.
 *
 * @param  {Object}   $      Object containing a reference to autoloaded gulp plugins
 * @param  {Function} gulp   Reference to the current instance of gulp
 * @return {Function}        Helper to get the correct gulp sub task
 */
var task = function task($, gulp) {
    return function (task, opts) {
        // Try and just get the task name first
        try {
            return require('./' + task)($, gulp, opts);
        } catch (e) {
            console.log(e);
            // It has a sub method, so we need to split up the task name,
            // get the first part and then use the second part for the function
            var temp = task.split('.');
            if (typeof temp[1] === 'undefined') {
                throw new Error('Function not found');
            }
            return require('./' + temp[0])[temp[1]]($, gulp, opts);
        }
    };
};

/**
 * Overwrite `gulp.src` to catch errors with `gulp-plumber` and `gulp-notify`.
 * @param  {Function} gulp   Reference to the current instance of gulp
 */
var niceErrors = function niceErrors($, gulp) {
    var _gulpsrc = gulp.src;
    gulp.src = function () {
        return _gulpsrc.apply(gulp, arguments).pipe($.plumber({
            errorHandler: function errorHandler(err) {
                $.notify.onError({
                    title: 'Gulp Error',
                    message: 'Error: <%= error.message %>',
                    sound: 'Frog'
                })(err);
                this.emit('end');
            }
        }));
    };
};

module.exports = {
    defaults: deepObjectExtend,
    niceErrors: niceErrors,
    task: task
};